# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 Pekka Helenius
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Style editor to edit Link items.
"""

from ..qt import QtCore, QtWidgets, QtGui
from ..ui.style_editor_dialog_ui import Ui_StyleEditorDialog


class StyleEditorDialogLink(QtWidgets.QDialog, Ui_StyleEditorDialog):

    """
    Style editor dialog.

    :param parent: parent widget
    :param link: selected link
    """

    def __init__(self, link, parent):

        super().__init__(parent)
        self.setupUi(self)

        self._link = link
        self._link_style = {}

        self.uiBorderColorPushButton.clicked.connect(self._setBorderColorSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Apply).clicked.connect(self._applyPreferencesSlot)

        self.uiBorderStyleComboBox.addItem("Solid", QtCore.Qt.SolidLine)
        self.uiBorderStyleComboBox.addItem("Dash", QtCore.Qt.DashLine)
        self.uiBorderStyleComboBox.addItem("Dot", QtCore.Qt.DotLine)
        self.uiBorderStyleComboBox.addItem("Dash Dot", QtCore.Qt.DashDotLine)
        self.uiBorderStyleComboBox.addItem("Dash Dot Dot", QtCore.Qt.DashDotDotLine)
        self.uiBorderStyleComboBox.addItem("Invisible", QtCore.Qt.NoPen)

        self.uiColorLabel.hide()
        self.uiColorPushButton.hide()
        self._color = None

        self.uiCornerRadiusLabel.hide()
        self.uiCornerRadiusSpinBox.hide()
        self.uiRotationLabel.hide()
        self.uiRotationSpinBox.hide()

        link.setHovered(False)  # make sure we use the right style
        pen = link.pen()
        link.setHovered(True)

        self._border_color = pen.color()
        self.uiBorderColorPushButton.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self._border_color.red(),
                                                                                                    self._border_color.green(),
                                                                                                    self._border_color.blue(),
                                                                                                    self._border_color.alpha()))
        self.uiBorderWidthSpinBox.setValue(pen.width())
        index = self.uiBorderStyleComboBox.findData(pen.style())
        if index != -1:
            self.uiBorderStyleComboBox.setCurrentIndex(index)

        self.adjustSize()

    def _setBorderColorSlot(self):
        """
        Slot to select the border color.
        """

        color = QtWidgets.QColorDialog.getColor(self._border_color, self, "Select Color", QtWidgets.QColorDialog.ShowAlphaChannel)
        if color.isValid():
            self._border_color = color
            self.uiBorderColorPushButton.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(self._border_color.red(),
                                                                                                        self._border_color.green(),
                                                                                                        self._border_color.blue(),
                                                                                                        self._border_color.alpha()))

    def _applyPreferencesSlot(self):
        """
        Applies the new style settings.
        """

        border_style = QtCore.Qt.PenStyle(self.uiBorderStyleComboBox.itemData(self.uiBorderStyleComboBox.currentIndex()))
        pen = QtGui.QPen(self._border_color, self.uiBorderWidthSpinBox.value(), border_style, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)

        self._link.setPen(pen)

        new_link_style = {}
        new_link_style["color"] = self._border_color.name()
        new_link_style["width"] = self.uiBorderWidthSpinBox.value()
        new_link_style["type"]  = border_style
        
        # Store values
        self._link.setLinkStyle(new_link_style)
        self._link.setHovered(False)  # allow to see the new style

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """

        if result:
            self._applyPreferencesSlot()
        super().done(result)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Functions to start SPICE console programs.
"""

import sys
import os
import shlex
import subprocess

from .controller import Controller

import logging
log = logging.getLogger(__name__)


def spiceConsole(node, port, command):
    """
    Start a SPICE console program.

    :param node: Node instance
    :param port: port number
    :param command: command to be executed
    """

    if len(command.strip(' ')) == 0:
        log.error("SPICE client is not configured")
        return

    name = node.name()
    host = node.consoleHost()

    # ipv6 support
    if ":" in host:
        host = "[{}]".format(host)

    # replace the place-holders by the actual values
    command = command.replace("%h", host)
    command = command.replace("%p", str(port))
    command = command.replace("%d", name.replace('"', '\\"'))
    command = command.replace("%P", node.project().name().replace('"', '\\"'))
    command = command.replace("%i", node.project().id())
    command = command.replace("%n", str(node.id()))
    command = command.replace("%c", Controller.instance().httpClient().fullUrl())

    command = command.replace("{host}", host)
    command = command.replace("{port}", str(port))
    command = command.replace("{name}", name.replace('"', '\\"'))
    command = command.replace("{project}", node.project().name())
    command = command.replace("{project_id}", node.project().id())
    command = command.replace("{node_id}", str(node.id()))
    command = command.replace("{url}", Controller.instance().httpClient().fullUrl())

    try:
        log.debug('starting SPICE program "{}"'.format(command))
        if sys.platform.startswith("win"):
            # use the string on Windows
            subprocess.Popen(command, env=os.environ)
        else:
            # use arguments on other platforms
            args = shlex.split(command)
            subprocess.Popen(args, env=os.environ)
    except (OSError, ValueError, subprocess.SubprocessError) as e:
        log.error("Could not start SPICE program with command '{}': {}".format(command, e))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for QEMU VMs.
"""

import os
import re

from collections import OrderedDict
from gns3.modules.qemu.dialogs.qemu_image_wizard import QemuImageWizard
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from gns3.ports.port_name_factory import StandardPortNameFactory
from gns3.dialogs.custom_adapters_configuration_dialog import CustomAdaptersConfigurationDialog
from gns3.node import Node
from gns3.qt import QtCore, QtWidgets, qpartial, sip_is_deleted
from gns3.dialogs.node_properties_dialog import ConfigurationError
from gns3.image_manager import ImageManager

from ..ui.qemu_vm_configuration_page_ui import Ui_QemuVMConfigPageWidget
from .. import Qemu


class QemuVMConfigurationPage(QtWidgets.QWidget, Ui_QemuVMConfigPageWidget):
    """
    QWidget configuration page for QEMU VMs.
    """

    _default_images_dir = ""

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self._compute_id = None
        self._settings = None
        self._custom_adapters = []

        self.uiQemuPlatformComboBox.addItems(Qemu.getQemuPlatforms())

        self.uiBootPriorityComboBox.addItem("HDD", "c")
        self.uiBootPriorityComboBox.addItem("CD/DVD-ROM", "d")
        self.uiBootPriorityComboBox.addItem("Network", "n")
        self.uiBootPriorityComboBox.addItem("HDD or Network", "cn")
        self.uiBootPriorityComboBox.addItem("HDD or CD/DVD-ROM", "cd")
        self.uiBootPriorityComboBox.addItem("CD/DVD-ROM or Network", "dn")
        self.uiBootPriorityComboBox.addItem("CD/DVD-ROM or HDD", "dc")
        self.uiBootPriorityComboBox.addItem("Network or HDD", "nc")
        self.uiBootPriorityComboBox.addItem("Network or CD/DVD-ROM", "nd")

        self.uiHdaDiskImageToolButton.clicked.connect(self._hdaDiskImageBrowserSlot)
        self.uiHdbDiskImageToolButton.clicked.connect(self._hdbDiskImageBrowserSlot)
        self.uiHdcDiskImageToolButton.clicked.connect(self._hdcDiskImageBrowserSlot)
        self.uiHddDiskImageToolButton.clicked.connect(self._hddDiskImageBrowserSlot)
        self.uiCdromImageToolButton.clicked.connect(self._cdromImageBrowserSlot)
        self.uiBiosImageToolButton.clicked.connect(self._biosImageBrowserSlot)

        self.uiHdaDiskImageCreateToolButton.clicked.connect(self._hdaDiskImageCreateSlot)
        self.uiHdbDiskImageCreateToolButton.clicked.connect(self._hdbDiskImageCreateSlot)
        self.uiHdcDiskImageCreateToolButton.clicked.connect(self._hdcDiskImageCreateSlot)
        self.uiHddDiskImageCreateToolButton.clicked.connect(self._hddDiskImageCreateSlot)

        self.uiHdaDiskImageResizeToolButton.clicked.connect(self._hdaDiskImageResizeSlot)
        self.uiHdbDiskImageResizeToolButton.clicked.connect(self._hdbDiskImageResizeSlot)
        self.uiHdcDiskImageResizeToolButton.clicked.connect(self._hdcDiskImageResizeSlot)
        self.uiHddDiskImageResizeToolButton.clicked.connect(self._hddDiskImageResizeSlot)

        disk_interfaces = ["ide", "sata", "nvme", "scsi", "sd", "mtd", "floppy", "pflash", "virtio", "none"]
        self.uiHdaDiskInterfaceComboBox.addItems(disk_interfaces)
        self.uiHdbDiskInterfaceComboBox.addItems(disk_interfaces)
        self.uiHdcDiskInterfaceComboBox.addItems(disk_interfaces)
        self.uiHddDiskInterfaceComboBox.addItems(disk_interfaces)

        self.uiSymbolToolButton.clicked.connect(self._symbolBrowserSlot)
        self.uiInitrdToolButton.clicked.connect(self._initrdBrowserSlot)
        self.uiKernelImageToolButton.clicked.connect(self._kernelImageBrowserSlot)
        self.uiActivateCPUThrottlingCheckBox.stateChanged.connect(self._cpuThrottlingChangedSlot)
        self.uiCustomAdaptersConfigurationPushButton.clicked.connect(self._customAdaptersConfigurationSlot)
        self.uiCreateConfigDiskCheckBox.stateChanged.connect(self._createConfigDiskChangedSlot)

        # add the categories
        for name, category in Node.defaultCategories().items():
            self.uiCategoryComboBox.addItem(name, category)

        # add the on close options
        for name, option_name in Node.onCloseOptions().items():
            self.uiOnCloseComboBox.addItem(name, option_name)

        # Supported NIC models: e1000, e1000-82544gc, e1000-82545em, e1000e, i82550, i82551, i82557a, i82557b, i82557c, i82558a
        # i82558b, i82559a, i82559b, i82559c, i82559er, i82562, i82801, igb, ne2k_pci, pcnet, rocker, rtl8139, virtio-net-pci, vmxnet3
        # This list can be retrieved using "qemu-system-x86_64 -nic model=?" or "qemu-system-x86_64 -device help"
        self._qemu_network_devices = OrderedDict([("e1000", "Intel Gigabit Ethernet"),
                                                  ("e1000-82544gc", "Intel 82544GC Gigabit Ethernet"),
                                                  ("e1000-82545em", "Intel 82545EM Gigabit Ethernet"),
                                                  ("e1000e", "Intel PCIe Gigabit Ethernet"),
                                                  ("i82550", "Intel i82550 Ethernet"),
                                                  ("i82551", "Intel i82551 Ethernet"),
                                                  ("i82557a", "Intel i82557A Ethernet"),
                                                  ("i82557b", "Intel i82557B Ethernet"),
                                                  ("i82557c", "Intel i82557C Ethernet"),
                                                  ("i82558a", "Intel i82558A Ethernet"),
                                                  ("i82558b", "Intel i82558B Ethernet"),
                                                  ("i82559a", "Intel i82559A Ethernet"),
                                                  ("i82559b", "Intel i82559B Ethernet"),
                                                  ("i82559c", "Intel i82559C Ethernet"),
                                                  ("i82559er", "Intel i82559ER Ethernet"),
                                                  ("i82562", "Intel i82562 Ethernet"),
                                                  ("i82801", "Intel i82801 Ethernet"),
                                                  ("igb", "Intel 82576 Gigabit Ethernet"),
                                                  ("ne2k_pci", "NE2000 Ethernet"),
                                                  ("pcnet", "AMD PCNet Ethernet"),
                                                  ("rocker", "Rocker L2 switch device"),
                                                  ("rtl8139", "Realtek 8139 Ethernet"),
                                                  ("virtio-net-pci", "Paravirtualized Network I/O"),
                                                  ("vmxnet3", "VMWare Paravirtualized Ethernet v3")])

        self._refreshQemuNetworkDevices()

    def _symbolBrowserSlot(self):
        """
        Slot to open the symbol browser and select a new symbol.
        """

        symbol_path = self.uiSymbolLineEdit.text()
        dialog = SymbolSelectionDialog(self, symbol=symbol_path)
        dialog.show()
        if dialog.exec_():
            new_symbol_path = dialog.getSymbol()
            self.uiSymbolLineEdit.setText(new_symbol_path)
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(new_symbol_path))

    def _refreshQemuNetworkDevices(self,):
        """
        Refreshes the Qemu network device list.

        """

        self.uiAdapterTypesComboBox.clear()
        for device_name, device_description in self._qemu_network_devices.items():
            self.uiAdapterTypesComboBox.addItem("{} ({})".format(device_description, device_name), device_name)

    @staticmethod
    def getImageDirectory():
        return ImageManager.instance().getDirectoryForType("QEMU")

    @classmethod
    def getDiskImage(cls, parent, server):

        if not cls._default_images_dir:
            cls._default_images_dir = cls.getImageDirectory()

        path, _ = QtWidgets.QFileDialog.getOpenFileName(parent, "Select a QEMU disk image", cls._default_images_dir)
        if not path:
            return
        cls._default_images_dir = os.path.dirname(path)

        if not os.access(path, os.R_OK):
            QtWidgets.QMessageBox.critical(parent, "QEMU disk image", "Cannot read {}".format(path))
            return

        path = ImageManager.instance().askCopyUploadImage(parent, path, server, "QEMU")

        return path

    def _hdaDiskImageBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU hda disk image.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiHdaDiskImageLineEdit.clear()
            self.uiHdaDiskImageLineEdit.setText(path)

    def _hdbDiskImageBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU hdb disk image.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiHdbDiskImageLineEdit.clear()
            self.uiHdbDiskImageLineEdit.setText(path)

    def _hdcDiskImageBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU hdc disk image.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiHdcDiskImageLineEdit.clear()
            self.uiHdcDiskImageLineEdit.setText(path)

    def _hddDiskImageBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU hdd disk image.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiHddDiskImageLineEdit.clear()
            self.uiHddDiskImageLineEdit.setText(path)

    def _biosImageBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU bios image.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiBiosImageLineEdit.clear()
            self.uiBiosImageLineEdit.setText(path)

    def _cdromImageBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU CD/DVD-ROM image.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiCdromImageLineEdit.clear()
            self.uiCdromImageLineEdit.setText(path)

    def _hdaDiskImageCreateSlot(self):

        if self._node:
            create_dialog = QemuImageWizard(self, self._node, self.uiNameLineEdit.text() + '-hda')
            if QtWidgets.QDialog.Accepted == create_dialog.exec_():
                self.uiHdaDiskImageLineEdit.setText(create_dialog.uiDiskFilenameLineEdit.text())

    def _hdbDiskImageCreateSlot(self):

        if self._node:
            create_dialog = QemuImageWizard(self, self._node, self.uiNameLineEdit.text() + '-hdb')
            if QtWidgets.QDialog.Accepted == create_dialog.exec_():
                self.uiHdbDiskImageLineEdit.setText(create_dialog.uiDiskFilenameLineEdit.text())

    def _hdcDiskImageCreateSlot(self):

        if self._node:
            create_dialog = QemuImageWizard(self, self._node, self.uiNameLineEdit.text() + '-hdc')
            if QtWidgets.QDialog.Accepted == create_dialog.exec_():
                self.uiHdcDiskImageLineEdit.setText(create_dialog.uiDiskFilenameLineEdit.text())

    def _hddDiskImageCreateSlot(self):

        if self._node:
            create_dialog = QemuImageWizard(self, self._node, self.uiNameLineEdit.text() + '-hdd')
            if QtWidgets.QDialog.Accepted == create_dialog.exec_():
                self.uiHddDiskImageLineEdit.setText(create_dialog.uiDiskFilenameLineEdit.text())

    def _hdaDiskImageResizeSlot(self):

        disk_image_filename = self.uiHdaDiskImageLineEdit.text().strip()
        if disk_image_filename:
            size, ok = QtWidgets.QInputDialog.getInt(self, "HDA disk size", "Increase hda disk size in MB:", 10000, 1, 1000000000, 1000)
            if ok and self._node:
                self._node.resizeDiskImage(disk_image_filename, size, self._resizeDiskImageCallback)

    def _hdbDiskImageResizeSlot(self):

        disk_image_filename = self.uiHdbDiskImageLineEdit.text()
        if disk_image_filename:
            size, ok = QtWidgets.QInputDialog.getInt(self, "HDB disk size", "Increase hdb disk size in MB:", 10000, 1, 1000000000, 1000)
            if ok and self._node:
                self._node.resizeDiskImage(disk_image_filename, size, self._resizeDiskImageCallback)

    def _hdcDiskImageResizeSlot(self):

        disk_image_filename = self.uiHdcDiskImageLineEdit.text()
        if disk_image_filename:
            size, ok = QtWidgets.QInputDialog.getInt(self, "HDC disk size", "Increase hdc disk size in MB:", 10000, 1, 1000000000, 1000)
            if ok and self._node:
                self._node.resizeDiskImage(disk_image_filename, size, self._resizeDiskImageCallback)

    def _hddDiskImageResizeSlot(self):

        disk_image_filename = self.uiHddDiskImageLineEdit.text()
        if disk_image_filename:
            size, ok = QtWidgets.QInputDialog.getInt(self, "HDD disk size", "Increase hdd disk size in MB:", 10000, 1, 1000000000, 1000)
            if ok and self._node:
                self._node.resizeDiskImage(disk_image_filename, size, self._resizeDiskImageCallback)

    def _resizeDiskImageCallback(self, result, error=False, **kwargs):
        """
        Callback for resizing a disk image in a VM.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "Disk image", "{}".format(result["message"]))
        else:
            QtWidgets.QMessageBox.information(self, "Disk image", "The disk has been resized")


    def _initrdBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU initrd.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiInitrdLineEdit.clear()
            self.uiInitrdLineEdit.setText(path)

    def _kernelImageBrowserSlot(self):
        """
        Slot to open a file browser and select a QEMU kernel image.
        """

        path = self.getDiskImage(self, self._compute_id)
        if path:
            self.uiKernelImageLineEdit.clear()
            self.uiKernelImageLineEdit.setText(path)


    def _cpuThrottlingChangedSlot(self, state):
        """
        Slot to enable or not CPU throttling.
        """

        if state:
            self.uiCPUThrottlingSpinBox.setEnabled(True)
        else:
            self.uiCPUThrottlingSpinBox.setEnabled(False)

    def _createConfigDiskChangedSlot(self, state):
        """
        Slot to allow or not HDD disk to be configured based on the state of the config disk option.
        """

        _translate = QtCore.QCoreApplication.translate
        if state:
            self.uiHddDiskImageLabel.setText(_translate("QemuVMConfigPageWidget", "Startup-cfg:"))
        else:
            self.uiHddDiskImageLabel.setText(_translate("QemuVMConfigPageWidget", "Disk image:"))
        self.uiHddDiskImageCreateToolButton.setEnabled(not state)
        self.uiHddDiskImageResizeToolButton.setEnabled(not state)

    def _customAdaptersConfigurationSlot(self):
        """
        Slot to open the custom adapters configuration dialog
        """

        if self._node:
            first_port_name = self._settings["first_port_name"]
            port_segment_size = self._settings["port_segment_size"]
            port_name_format = self._settings["port_name_format"]
            adapters = self._settings["adapters"]
            default_adapter = self._settings["adapter_type"]
            base_mac_address = self._settings["mac_address"]
        else:
            first_port_name = self.uiFirstPortNameLineEdit.text().strip()
            port_name_format = self.uiPortNameFormatLineEdit.text()
            port_segment_size = self.uiPortSegmentSizeSpinBox.value()
            adapters = self.uiAdaptersSpinBox.value()
            default_adapter = self.uiAdapterTypesComboBox.currentData()

            mac = self.uiMacAddrLineEdit.text()
            if mac != ":::::":
                if not re.search(r"""^([0-9a-fA-F]{2}[:]){5}[0-9a-fA-F]{2}$""", mac):
                    QtWidgets.QMessageBox.critical(self, "MAC address", "Invalid MAC address (format required: hh:hh:hh:hh:hh:hh)")
                    return
                else:
                    base_mac_address = mac
            else:
                base_mac_address = ""

        try:
            ports = StandardPortNameFactory(adapters, first_port_name, port_name_format, port_segment_size)
        except (IndexError, ValueError, KeyError):
            QtWidgets.QMessageBox.critical(self, "Invalid format", "Invalid port name format")
            return

        network_devices = self._qemu_network_devices.copy()
        dialog = CustomAdaptersConfigurationDialog(ports, self._custom_adapters, default_adapter, network_devices, base_mac_address, parent=self)
        dialog.show()
        dialog.exec_()

    def loadSettings(self, settings, node=None, group=False):
        """
        Loads the QEMU VM settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of VMs
        """

        if node:
            self._compute_id = node.compute().id()
            self._node = node
            self._settings = settings
        else:
            self._compute_id = settings["compute_id"]
            self._node = None

        if not group:
            # set the device name
            self.uiNameLineEdit.setText(settings["name"])

            if "linked_clone" in settings:
                self.uiBaseVMCheckBox.setChecked(settings["linked_clone"])
            else:
                self.uiBaseVMCheckBox.hide()

            self.uiHdaDiskImageLineEdit.setText(settings["hda_disk_image"])
            self.uiHdbDiskImageLineEdit.setText(settings["hdb_disk_image"])
            self.uiHdcDiskImageLineEdit.setText(settings["hdc_disk_image"])
            self.uiHddDiskImageLineEdit.setText(settings["hdd_disk_image"])
            self.uiHdaDiskInterfaceComboBox.setCurrentIndex(self.uiHdaDiskInterfaceComboBox.findText(settings["hda_disk_interface"]))
            self.uiHdbDiskInterfaceComboBox.setCurrentIndex(self.uiHdbDiskInterfaceComboBox.findText(settings["hdb_disk_interface"]))
            self.uiHdcDiskInterfaceComboBox.setCurrentIndex(self.uiHdcDiskInterfaceComboBox.findText(settings["hdc_disk_interface"]))
            self.uiHddDiskInterfaceComboBox.setCurrentIndex(self.uiHddDiskInterfaceComboBox.findText(settings["hdd_disk_interface"]))
            self.uiCreateConfigDiskCheckBox.setChecked(settings["create_config_disk"])
            self.uiCdromImageLineEdit.setText(settings["cdrom_image"])
            self.uiBiosImageLineEdit.setText(settings["bios_image"])
            self.uiInitrdLineEdit.setText(settings["initrd"])
            self.uiKernelImageLineEdit.setText(settings["kernel_image"])
        else:
            self.uiNameLabel.hide()
            self.uiNameLineEdit.hide()
            self.uiHddTab.hide()
            self.uiCdromTab.hide()
            self.uiBiosImageGroupBox.hide()
            self.uiInitrdLabel.hide()
            self.uiInitrdLineEdit.hide()
            self.uiInitrdToolButton.hide()
            self.uiKernelImageLabel.hide()
            self.uiKernelImageLineEdit.hide()
            self.uiKernelImageToolButton.hide()

        if not node:
            # these are template settings

            self.uiNameLabel.setText("Template name:")

            # load the default name format
            self.uiDefaultNameFormatLineEdit.setText(settings["default_name_format"])

            # load the symbol
            self.uiSymbolLineEdit.setText(settings["symbol"])
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(settings["symbol"]))

            # load the category
            index = self.uiCategoryComboBox.findData(settings["category"])
            if index != -1:
                self.uiCategoryComboBox.setCurrentIndex(index)

            self.uiPortNameFormatLineEdit.setText(settings["port_name_format"])
            self.uiPortSegmentSizeSpinBox.setValue(settings["port_segment_size"])
            self.uiFirstPortNameLineEdit.setText(settings["first_port_name"])

            self.uiHdaDiskImageCreateToolButton.hide()
            self.uiHdbDiskImageCreateToolButton.hide()
            self.uiHdcDiskImageCreateToolButton.hide()
            self.uiHddDiskImageCreateToolButton.hide()

            self.uiHdaDiskImageResizeToolButton.hide()
            self.uiHdbDiskImageResizeToolButton.hide()
            self.uiHdcDiskImageResizeToolButton.hide()
            self.uiHddDiskImageResizeToolButton.hide()
        else:
            self.uiDefaultNameFormatLabel.hide()
            self.uiDefaultNameFormatLineEdit.hide()
            self.uiSymbolLabel.hide()
            self.uiSymbolLineEdit.hide()
            self.uiSymbolToolButton.hide()
            self.uiCategoryComboBox.hide()
            self.uiCategoryLabel.hide()
            self.uiCategoryComboBox.hide()
            self.uiPortNameFormatLabel.hide()
            self.uiPortNameFormatLineEdit.hide()
            self.uiPortSegmentSizeLabel.hide()
            self.uiPortSegmentSizeSpinBox.hide()
            self.uiFirstPortNameLabel.hide()
            self.uiFirstPortNameLineEdit.hide()

        index = self.uiQemuPlatformComboBox.findText(settings["platform"])
        if index != -1:
            self.uiQemuPlatformComboBox.setCurrentIndex(index)

        index = self.uiBootPriorityComboBox.findData(settings["boot_priority"])
        if index != -1:
            self.uiBootPriorityComboBox.setCurrentIndex(index)

        index = self.uiConsoleTypeComboBox.findText(settings["console_type"])
        if index != -1:
            self.uiConsoleTypeComboBox.setCurrentIndex(index)

        index = self.uiAuxTypeComboBox.findText(settings["aux_type"])
        if index != -1:
            self.uiAuxTypeComboBox.setCurrentIndex(index)

        self.uiConsoleAutoStartCheckBox.setChecked(settings["console_auto_start"])
        self.uiKernelCommandLineEdit.setText(settings["kernel_command_line"])
        self.uiAdaptersSpinBox.setValue(settings["adapters"])
        self._custom_adapters = settings["custom_adapters"].copy()
        self.uiReplicateNetworkConnectionStateCheckBox.setChecked(settings["replicate_network_connection_state"])

        # load the MAC address setting
        self.uiMacAddrLineEdit.setInputMask("HH:HH:HH:HH:HH:HH;_")
        if settings["mac_address"]:
            self.uiMacAddrLineEdit.setText(settings["mac_address"])
        else:
            self.uiMacAddrLineEdit.clear()

        # load the on close option
        index = self.uiOnCloseComboBox.findData(settings["on_close"])
        if index != -1:
            self.uiOnCloseComboBox.setCurrentIndex(index)

        index = self.uiAdapterTypesComboBox.findData(settings["adapter_type"])
        if index != -1:
            self.uiAdapterTypesComboBox.setCurrentIndex(index)

        self.uiCPUSpinBox.setValue(settings["cpus"])
        self.uiRamSpinBox.setValue(settings["ram"])

        if settings["cpu_throttling"]:
            self.uiActivateCPUThrottlingCheckBox.setChecked(True)
            self.uiCPUThrottlingSpinBox.setValue(settings["cpu_throttling"])
        else:
            self.uiActivateCPUThrottlingCheckBox.setChecked(False)

        index = self.uiProcessPriorityComboBox.findText(settings["process_priority"], QtCore.Qt.MatchFixedString)
        if index != -1:
            self.uiProcessPriorityComboBox.setCurrentIndex(index)
        self.uiQemuPathLineEdit.setText(settings["qemu_path"])
        self.uiQemuOptionsLineEdit.setText(settings["options"])
        self.uiUsageTextEdit.setPlainText(settings["usage"])
        self.uiTPMCheckBox.setChecked(settings["tpm"])
        self.uiUEFICheckBox.setChecked(settings["uefi"])

    def saveSettings(self, settings, node=None, group=False):
        """
        Saves the QEMU VM settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of VMs
        """

        # these settings cannot be shared by nodes and updated
        # in the node properties dialog.
        if not group:

            name = self.uiNameLineEdit.text()
            if not name:
                QtWidgets.QMessageBox.critical(self, "Name", "The QEMU VM name cannot be empty!")
            else:
                settings["name"] = name

            if "linked_clone" in settings:
                settings["linked_clone"] = self.uiBaseVMCheckBox.isChecked()

            settings["hda_disk_image"] = self.uiHdaDiskImageLineEdit.text().strip()
            settings["hdb_disk_image"] = self.uiHdbDiskImageLineEdit.text().strip()
            settings["hdc_disk_image"] = self.uiHdcDiskImageLineEdit.text().strip()
            settings["hdd_disk_image"] = self.uiHddDiskImageLineEdit.text().strip()
            settings["hda_disk_interface"] = self.uiHdaDiskInterfaceComboBox.currentText()
            settings["hdb_disk_interface"] = self.uiHdbDiskInterfaceComboBox.currentText()
            settings["hdc_disk_interface"] = self.uiHdcDiskInterfaceComboBox.currentText()
            settings["hdd_disk_interface"] = self.uiHddDiskInterfaceComboBox.currentText()
            settings["create_config_disk"] = self.uiCreateConfigDiskCheckBox.isChecked()
            settings["cdrom_image"] = self.uiCdromImageLineEdit.text().strip()
            settings["bios_image"] = self.uiBiosImageLineEdit.text().strip()
            settings["initrd"] = self.uiInitrdLineEdit.text().strip()
            settings["kernel_image"] = self.uiKernelImageLineEdit.text().strip()

            # check and save the MAC address
            mac = self.uiMacAddrLineEdit.text()
            if mac != ":::::":
                if not re.search(r"""^([0-9a-fA-F]{2}[:]){5}[0-9a-fA-F]{2}$""", mac):
                    QtWidgets.QMessageBox.critical(self, "MAC address", "Invalid MAC address (format required: hh:hh:hh:hh:hh:hh)")
                    if node:
                        raise ConfigurationError()
                else:
                    settings["mac_address"] = mac
            else:
                settings["mac_address"] = None

        if not node:
            # these are template settings

            # save the default name format
            default_name_format = self.uiDefaultNameFormatLineEdit.text().strip()
            if '{0}' not in default_name_format and '{id}' not in default_name_format:
                QtWidgets.QMessageBox.critical(self, "Default name format", "The default name format must contain at least {0} or {id}")
            else:
                settings["default_name_format"] = default_name_format

            symbol_path = self.uiSymbolLineEdit.text()
            settings["symbol"] = symbol_path
            settings["category"] = self.uiCategoryComboBox.itemData(self.uiCategoryComboBox.currentIndex())

            port_name_format = self.uiPortNameFormatLineEdit.text()
            port_segment_size = self.uiPortSegmentSizeSpinBox.value()
            first_port_name = self.uiFirstPortNameLineEdit.text().strip()

            try:
                StandardPortNameFactory(self.uiAdaptersSpinBox.value(), first_port_name, port_name_format, port_segment_size)
            except (IndexError, ValueError, KeyError):
                QtWidgets.QMessageBox.critical(self, "Invalid format", "Invalid port name format")
                raise ConfigurationError()

            settings["port_name_format"] = self.uiPortNameFormatLineEdit.text()
            settings["port_segment_size"] = port_segment_size
            settings["first_port_name"] = first_port_name

        settings["platform"] = self.uiQemuPlatformComboBox.itemText(self.uiQemuPlatformComboBox.currentIndex())
        settings["boot_priority"] = self.uiBootPriorityComboBox.itemData(self.uiBootPriorityComboBox.currentIndex())
        settings["console_type"] = self.uiConsoleTypeComboBox.currentText().lower()
        settings["console_auto_start"] = self.uiConsoleAutoStartCheckBox.isChecked()
        settings["aux_type"] = self.uiAuxTypeComboBox.currentText().lower()
        settings["adapter_type"] = self.uiAdapterTypesComboBox.itemData(self.uiAdapterTypesComboBox.currentIndex())
        settings["kernel_command_line"] = self.uiKernelCommandLineEdit.text()

        adapters = self.uiAdaptersSpinBox.value()
        if node and node.settings()["adapters"] != adapters:
            # check if the adapters settings have changed
            node_ports = node.ports()
            for node_port in node_ports:
                if not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, node.name(), "Changing the number of adapters while links are connected isn't supported yet! Please delete all the links first.")
                    raise ConfigurationError()

        settings["adapters"] = adapters
        settings["replicate_network_connection_state"] = self.uiReplicateNetworkConnectionStateCheckBox.isChecked()
        settings["custom_adapters"] = self._custom_adapters.copy()
        settings["on_close"] = self.uiOnCloseComboBox.itemData(self.uiOnCloseComboBox.currentIndex())
        settings["cpus"] = self.uiCPUSpinBox.value()
        settings["ram"] = self.uiRamSpinBox.value()
        if self.uiActivateCPUThrottlingCheckBox.isChecked():
            settings["cpu_throttling"] = self.uiCPUThrottlingSpinBox.value()
        else:
            settings["cpu_throttling"] = 0
        settings["process_priority"] = self.uiProcessPriorityComboBox.currentText().lower()
        settings["qemu_path"] = self.uiQemuPathLineEdit.text().strip()
        settings["options"] = self.uiQemuOptionsLineEdit.text()
        settings["usage"] = self.uiUsageTextEdit.toPlainText()
        settings["tpm"] = self.uiTPMCheckBox.isChecked()
        settings["uefi"] = self.uiUEFICheckBox.isChecked()
        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VPCS node implementation.
"""

from gns3.node import Node

import logging
log = logging.getLogger(__name__)


class VPCSNode(Node):
    """
    VPCS node.

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """

    URL_PREFIX = "vpcs"

    def __init__(self, module, server, project):
        super().__init__(module, server, project)

        vpcs_settings = {"startup_script": None,
                         "console_type": "telnet",
                         "console_auto_start": True}

        self.settings().update(vpcs_settings)

    def info(self):
        """
        Returns information about this VPCS node.

        :returns: formatted string
        """

        info = """Node {name} is {state}
  Running on server {host} with port {port}
  Local ID is {id} and node ID is {node_id}
  Console is on port {console} and type is {console_type}
""".format(name=self.name(),
           id=self.id(),
           node_id=self._node_id,
           state=self.state(),
           host=self.compute().name(),
           port=self.compute().port(),
           console=self._settings["console"],
           console_type=self._settings["console_type"])

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "     {port_name} is empty\n".format(port_name=port.name())
            else:
                port_info += "     {port_name} {port_description}\n".format(port_name=port.name(),
                                                                            port_description=port.description())

        return info + port_info

    def configFiles(self):
        """
        Name of the configuration files
        """

        return ["startup.vpc"]

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.vpcs_node_configuration_page import VPCSNodeConfigurationPage
        return VPCSNodeConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """

        return ":/symbols/vpcs_guest.svg"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the node panel).

        :returns: list of node categories
        """

        return [Node.end_devices]

    def __str__(self):

        return "VPCS node"


# for compatibility pre version 2.0
class VPCSDevice(VPCSNode):
    pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from ..qt import QtCore, QtGui, QtWidgets
from ..ui.password_dialog_ui import Ui_PasswordDialog


import logging
log = logging.getLogger(__name__)


class PasswordDialog(QtWidgets.QDialog, Ui_PasswordDialog):

    """
    Password dialog.
    """

    def __init__(self, parent):
        """
        :param parent: parent widget.
        """

        super().__init__(parent)
        self.setupUi(self)
        self._password = None

        self._eye_on_icon = QtGui.QIcon(':/icons/eye-on.svg')
        self._eye_off_icon = QtGui.QIcon(':/icons/eye-off.svg')
        for line_edit in [self.uiPasswordLineEdit, self.uiConfirmPasswordLineEdit]:
            action = line_edit.addAction(self._eye_on_icon, QtWidgets.QLineEdit.TrailingPosition)
            button = action.associatedWidgets()[-1]
            button.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
            button.pressed.connect(self.onPressedSlot)
            #button.released.connect(self.onReleasedSlot)

    def onPressedSlot(self):

        button = self.sender()
        line_edit = button.parent()
        if line_edit.echoMode() == QtWidgets.QLineEdit.Password:
            button.setIcon(self._eye_off_icon)
            line_edit.setEchoMode(QtWidgets.QLineEdit.Normal)
        else:
            button.setIcon(self._eye_on_icon)
            line_edit.setEchoMode(QtWidgets.QLineEdit.Password)

    # def onReleasedSlot(self):
    #
    #     button = self.sender()
    #     button.setIcon(self._eye_on_icon)
    #     button.parent().setEchoMode(QtWidgets.QLineEdit.Password)

    def getPassword(self):

        return self._password

    def done(self, result):

        if result:
            new_password = self.uiPasswordLineEdit.text()
            confirm_password = self.uiConfirmPasswordLineEdit.text()
            if new_password != confirm_password:
                QtWidgets.QMessageBox.critical(self, "Error", "Passwords do not match.")
                return
            pattern = re.compile(r'^(?=.*\d)(?=.*[a-z])(?=.*[A-Z]).{8}$')
            if not pattern.match(new_password):
                QtWidgets.QMessageBox.critical(self, "Error", "Password must be at least 8 characters long and contain at least one digit, one lowercase letter and one uppercase letter.")
                return
            self._password = new_password
        super().done(result)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for user preferences.
"""

from gns3.qt import QtWidgets, qslot, sip_is_deleted
from ..ui.user_preferences_page_ui import Ui_UserPreferencesPageWidget
from gns3.controller import Controller
from gns3.dialogs.password_dialog import PasswordDialog

import logging
log = logging.getLogger(__name__)


class UserPreferencesPage(QtWidgets.QWidget, Ui_UserPreferencesPageWidget):

    """
    QWidget configuration page for user preferences.
    """

    def __init__(self, parent=None):

        super().__init__()
        self.setupUi(self)

        self.uiChangePasswordPushButton.clicked.connect(self._changePasswordSlot)
        self.uiCopyAccessTokenPushButton.clicked.connect(self._copyAccessTokenSlot)
        self.uiChangePasswordPushButton.setEnabled(False)
        self.uiCopyAccessTokenPushButton.setEnabled(False)
        Controller.instance().connected_signal.connect(self.loadPreferences)
        Controller.instance().disconnected_signal.connect(self._disconnectSlot)

    def _populateWidgets(self, result):
        """
        Populates the widgets with the user information.

        :param result: user information
        """

        self.uiUsernameLineEdit.setText(result.get("username", "N/A"))
        self.uiFullNameLineEdit.setText(result.get("full_name", "N/A"))
        self.uiEmailLineEdit.setText(result.get("email", "N/A"))
        self.uiChangePasswordPushButton.setEnabled(True)
        self.uiCopyAccessTokenPushButton.setEnabled(True)

    def _disconnectSlot(self):
        """
        Resets the widgets when the controller is disconnected.
        """

        self.uiUsernameLineEdit.clear()
        self.uiFullNameLineEdit.clear()
        self.uiEmailLineEdit.clear()
        self.uiChangePasswordPushButton.setEnabled(False)
        self.uiCopyAccessTokenPushButton.setEnabled(False)

    def loadPreferences(self):
        """
        Loads the user preferences.
        """

        if Controller.instance().connected():
            Controller.instance().get("/access/users/me", self._getUserInfoCallback)
        else:
            log.error("Cannot load the user information in the preferences dialog: not connected to the controller")

    @qslot
    def _getUserInfoCallback(self, result, error=False, **kwargs):
        """
        Callback to get the user information.
        """

        if sip_is_deleted(self):
            return
        if error:
            if "message" in result:
                log.error("Error while getting the logged-in user information: {}".format(result["message"]))
            return
        self._populateWidgets(result)

    def savePreferences(self):
        """
        Saves the user preferences.
        """

        pass

    def _changePasswordSlot(self):
        """
        Slot to change the user password.
        """

        dialog = PasswordDialog(self)
        if dialog.exec_():
            password = dialog.getPassword()
            new_settings = {"password": password}
            Controller.instance().put("/access/users/me", self._saveUserSettingsCallback, new_settings)

    def _saveUserSettingsCallback(self, result, error=False, **kwargs):
        """
        Callback to save the user settings.
        """

        if error and "message" in result:
            QtWidgets.QMessageBox.critical(
                self,
                "Save user settings",
                "Error while saving user settings: {}".format(result["message"])
            )
        else:
            QtWidgets.QMessageBox.information(self, "Change password", "New password saved successfully")

    def _copyAccessTokenSlot(self):
        """
        Slot to copy the access token to the clipboard.
        """

        if Controller.instance().connected():
            token = Controller.instance().httpClient().getToken()
            if token:
                QtWidgets.QApplication.clipboard().setText(token)
                log.info("Access token copied to the clipboard")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Compatibility layer for Qt bindings, so it is easier to switch to PySide if needed.
"""

# based on https://gist.github.com/remram44/5985681 and
# https://github.com/pyQode/pyqode.qt/blob/master/pyqode/qt/QtWidgets.py (MIT license)


import sys
import os
import re
import inspect
import functools

import logging
log = logging.getLogger("qt/__init__.py")

try:
    from PyQt5 import sip
except ImportError:
    import sip

from PyQt5 import QtCore, QtGui, QtNetwork, QtWidgets
sys.modules[__name__ + '.QtCore'] = QtCore
sys.modules[__name__ + '.QtGui'] = QtGui
sys.modules[__name__ + '.QtNetwork'] = QtNetwork
sys.modules[__name__ + '.QtWidgets'] = QtWidgets
sys.modules[__name__ + '.sip'] = sip

try:
    from PyQt5 import QtSvg
    sys.modules[__name__ + '.QtSvg'] = QtSvg
except ImportError:
    raise SystemExit("Please install the PyQt5.QtSvg module")

try:
    from PyQt5 import QtWebSockets
    sys.modules[__name__ + '.QtWebSockets'] = QtWebSockets
except ImportError:
    raise SystemExit("Please install the PyQt5.QtWebSockets module")

QtCore.Signal = QtCore.pyqtSignal
QtCore.Slot = QtCore.pyqtSlot
QtCore.Property = QtCore.pyqtProperty

from PyQt5.QtWidgets import QFileDialog as OldFileDialog
from PyQt5.QtNetwork import QNetworkAccessManager

# Do not use system proxy because it could be a parental control, virus or "Security software"...
QtNetwork.QNetworkProxyFactory.setUseSystemConfiguration(False)


def sip_is_deleted(obj):
    """
    :return: True if object no longer exists
    """
    if obj is None or (isinstance(obj, sip.simplewrapper) and sip.isdeleted(obj)):
        return True
    return False


class LogQMessageBox(QtWidgets.QMessageBox):
    """
    Replace the standard message box for logging errors to console. And
    show a stack trace when a critical message box is shown in debug mode
    """
    @staticmethod
    def critical(parent, title, message, *args, show_stack_info=False):
        if message.startswith("QXcbConnection"):  # Qt noise not relevant
            return
        stack_info = None
        if show_stack_info:
            stack_info = LogQMessageBox.stack_info()
        LogQMessageBox._get_logger().critical(re.sub(r"<[^<]+?>", "", message), stack_info=stack_info)
        if parent is False:
            # special case to display a QMessageBox before the main window is created.
            parent = None
        elif sip_is_deleted(parent):
            return
        return super(QtWidgets.QMessageBox, QtWidgets.QMessageBox).critical(parent, title, message, *args)

    @staticmethod
    def warning(parent, title, message, *args):
        LogQMessageBox._get_logger().warning(re.sub(r"<[^<]+?>", "", message))
        if parent is False:
            # special case to display a QMessageBox before the main window is created.
            parent = None
        elif sip_is_deleted(parent):
            return
        return super(QtWidgets.QMessageBox, QtWidgets.QMessageBox).warning(parent, title, message, *args)

    @staticmethod
    def _get_logger():
        """
        Return a logger in the context of the caller
        in order to have the correct information in the log
        """

        try:
            caller = inspect.stack()[2]
            location = "{}:{}".format(os.path.basename(caller.filename), caller.lineno)
        except Exception:
            # If anything go wrong during the format return the standard logger
            # for unknonw reason sometimes we don't have the caller info
            return logging.getLogger('qt')
        return logging.getLogger(location)

    @staticmethod
    def stack_info():
        """
        Show stack trace
        """
        return logging.getLogger().getEffectiveLevel() == logging.DEBUG

QtWidgets.QMessageBox = LogQMessageBox


# If we run from a test we replace the signal by a synchronous version
if hasattr(sys, '_called_from_test'):
    class FakeQtSignal:
        _instances = set()

        def __init__(self, *args):
            self._callbacks = set()
            self._instances.add(self)

        def connect(self, func, style=None):
            self._callbacks.add(func)

        def disconnect(self, func):
            self._callbacks.remove(func)

        def emit(self, *args):
            for callback in list(self._callbacks):
                callback(*args)

        @classmethod
        def reset(cls):
            """Use to clean the listening signals between tests"""
            for instance in cls._instances:
                instance._callbacks = set()

    QtCore.Signal = FakeQtSignal
    QtCore.pyqtSignal = FakeQtSignal


def qpartial(func, *args, **kwargs):
    """
    A functools partial that you can use on qobject. If the targeted qobject is
    destroyed the partial is not called.
    """
    if func is None:
        return None

    if inspect.ismethod(func):
        if isinstance(func.__self__, QtCore.QObject):

            def partial(*args, **kwargs):
                if sip_is_deleted(func.__self__):
                    return
                return func(*args, **kwargs)
            return functools.partial(partial, *args, **kwargs)

    return functools.partial(func, *args, **kwargs)


def qslot(func):
    """
    Decorated slot are protected against already destroyed element
    in SIP but not in Python
    """
    def func_wrapper(*args, **kwargs):
        if len(args) > 0:
            if sip_is_deleted(args[0]):
                return lambda: True
        return func(*args, **kwargs)
    return func_wrapper


# Log Qt messages to Python log
def myQtMsgHandler(msg_type, msg_log_context, msg_string):
    if "_COMPIZ_TOOLKIT_ACTION" in msg_string:
        # Qt < 5.6 issue: https://github.com/GNS3/gns3-gui/issues/2020
        return
    if msg_string.startswith("QXcbConnection"):  # Qt noise not relevant
        return
    log.debug(msg_string)


QtCore.qInstallMessageHandler(myQtMsgHandler)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class for node classes.
"""

from .qt import QtCore
from .ports.port import Port

import logging
log = logging.getLogger(__name__)


class BaseNode(QtCore.QObject):
    """
    BaseNode implementation.

    :param module: Module instance
    :param server: client connection to a server
    :param project: Project instance
    """

    # signals used to let the GUI know about some events.
    created_signal = QtCore.Signal(int)
    started_signal = QtCore.Signal()
    stopped_signal = QtCore.Signal()
    suspended_signal = QtCore.Signal()
    updated_signal = QtCore.Signal()
    loaded_signal = QtCore.Signal()
    deleted_signal = QtCore.Signal()
    error_signal = QtCore.Signal(int, str)
    server_error_signal = QtCore.Signal(int, str)

    _instance_count = 1

    # node statuses
    stopped = 0
    started = 1
    suspended = 2

    # node categories
    routers = "router"
    switches = "switch"
    end_devices = "guest"
    security_devices = "firewall"

    def __init__(self, module, compute, project):

        super().__init__()

        # create an unique ID
        self._id = BaseNode._instance_count
        BaseNode._instance_count += 1

        self._module = module
        self._compute = compute
        assert project is not None
        self._project = project
        self._initialized = False
        self._loading = False
        self._status = BaseNode.stopped
        self._ports = []
        self._links = set()

    def links(self):
        """
        Links connected to this node
        """

        return self._links

    def addLink(self, link):
        """
        Add a link connected to this node

        :param link: link object
        """

        self._links.add(link)

    def deleteLink(self, link):
        """
        Delete a link connected to this node

        :param link: link object
        """

        try:
            self._links.remove(link)
        except KeyError:
            pass

    def state(self):
        """
        Returns a human readable status of this node.

        :returns: string
        """

        status = self.status()
        if status == self.started:
            return "started"
        elif status == self.stopped:
            return "stopped"
        elif status == self.suspended:
            return "suspended"
        return "unknown"

    @classmethod
    def reset(cls):
        """
        Reset the instance count.
        """

        cls._instance_count = 1

    def module(self):
        """
        Returns this node module.

        :returns: Module instance
        """

        return self._module

    def compute(self):
        """
        Returns this node compute.

        :returns: Compute instance
        """
        return self._compute

    def project(self):
        """
        Returns this node project.

        :returns: Project instance
        """

        return self._project

    def id(self):
        """
        Returns this node identifier.

        :returns: node identifier (integer)
        """

        return self._id

    def setId(self, new_id):
        """
        Sets an identifier for this node.

        :param new_id: node identifier (integer)
        """

        self._id = new_id

        # update the instance count to avoid conflicts
        if new_id >= BaseNode._instance_count:
            BaseNode._instance_count = new_id + 1

    def status(self):
        """
        Returns the status of this node.
        0 = stopped, 1 = started, 2 = suspended.

        :returns: node status (integer)
        """

        return self._status

    def setStatus(self, status):
        """
        Sets a status for this node.
        0 = stopped, 1 = started, 2 = suspended.

        :param status: node status (integer)
        """

        if status == self._status:
            return
        self._status = status
        if status == self.started:
            for port in self._ports:
                # set ports as started
                port.setStatus(Port.started)
            self.started_signal.emit()
        elif status == self.stopped:
            for port in self._ports:
                # set ports as stopped
                port.setStatus(Port.stopped)
            self.stopped_signal.emit()
        elif status == self.suspended:
            for port in self._ports:
                # set ports as suspended
                port.setStatus(Port.suspended)
            self.suspended_signal.emit()

    def initialized(self):
        """
        Returns if the node has been initialized

        :returns: boolean
        """

        return self._initialized

    def setInitialized(self, initialized):
        """
        Sets if the node has been initialized

        :param initialized: boolean
        """

        self._initialized = initialized

    def update(self, new_settings):
        """
        Updates the settings for this node.
        Must be overloaded.

        :param new_settings: settings dictionary
        """

        raise NotImplementedError()

    def ports(self):
        """
        Returns all the ports for this node.

        :returns: list of Port instances
        """

        return self._ports

    def controllerHttpPost(self, path, callback, body={}, context={}, **kwargs):
        """
        POST on current server / project

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param body: params to send (dictionary)
        :param context: Pass a context to the response callback
        """

        self._project.post(path, callback, body=body, context=context, **kwargs)

    def controllerHttpPut(self, path, callback, body={}, context={}, **kwargs):
        """
        PUT on current server / project

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param body: params to send (dictionary)
        :param context: Pass a context to the response callback
        """

        self._project.put(path, callback, body=body, context=context, **kwargs)

    def controllerHttpGet(self, path, callback, context={}, **kwargs):
        """
        Get on current server / project

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param body: params to send (dictionary)
        :param context: Pass a context to the response callback
        """

        self._project.get(path, callback, context=context, **kwargs)

    def controllerHttpDelete(self, path, callback, context={}, **kwargs):
        """
        Delete on current server / project

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param context: Pass a context to the response callback
        """

        self._project.delete(path, callback, context=context, **kwargs)

    @staticmethod
    def defaultCategories():
        """
        Returns the default categories.

        :returns: dict
        """

        categories = {"Routers": BaseNode.routers,
                      "Switches": BaseNode.switches,
                      "End devices": BaseNode.end_devices,
                      "Security devices": BaseNode.security_devices}

        return categories

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.
        Must be overloaded.

        :returns: symbol path (or resource).
        """

        raise NotImplementedError()

    @staticmethod
    def categories(self):
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node category (integer)
        """

        raise NotImplementedError()

    def __str__(self):
        """
        Must be overloaded.
        """

        raise NotImplementedError()


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/user_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_UserPreferencesPageWidget(object):
    def setupUi(self, UserPreferencesPageWidget):
        UserPreferencesPageWidget.setObjectName("UserPreferencesPageWidget")
        UserPreferencesPageWidget.resize(472, 370)
        self.verticalLayout = QtWidgets.QVBoxLayout(UserPreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiSettingsGroupBox = QtWidgets.QGroupBox(UserPreferencesPageWidget)
        self.uiSettingsGroupBox.setObjectName("uiSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiSettingsGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiEmailLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiEmailLabel.setObjectName("uiEmailLabel")
        self.gridLayout.addWidget(self.uiEmailLabel, 3, 0, 1, 1)
        self.uiFullNameLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiFullNameLabel.setObjectName("uiFullNameLabel")
        self.gridLayout.addWidget(self.uiFullNameLabel, 2, 0, 1, 1)
        self.uiUsernameLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiUsernameLabel.setObjectName("uiUsernameLabel")
        self.gridLayout.addWidget(self.uiUsernameLabel, 0, 0, 1, 1)
        self.uiUsernameLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiUsernameLineEdit.setReadOnly(True)
        self.uiUsernameLineEdit.setObjectName("uiUsernameLineEdit")
        self.gridLayout.addWidget(self.uiUsernameLineEdit, 0, 1, 1, 1)
        self.uiFullNameLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiFullNameLineEdit.setReadOnly(True)
        self.uiFullNameLineEdit.setObjectName("uiFullNameLineEdit")
        self.gridLayout.addWidget(self.uiFullNameLineEdit, 2, 1, 1, 1)
        self.uiEmailLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiEmailLineEdit.setReadOnly(True)
        self.uiEmailLineEdit.setObjectName("uiEmailLineEdit")
        self.gridLayout.addWidget(self.uiEmailLineEdit, 3, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiSettingsGroupBox)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiChangePasswordPushButton = QtWidgets.QPushButton(UserPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiChangePasswordPushButton.sizePolicy().hasHeightForWidth())
        self.uiChangePasswordPushButton.setSizePolicy(sizePolicy)
        self.uiChangePasswordPushButton.setObjectName("uiChangePasswordPushButton")
        self.horizontalLayout.addWidget(self.uiChangePasswordPushButton)
        self.uiCopyAccessTokenPushButton = QtWidgets.QPushButton(UserPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiCopyAccessTokenPushButton.sizePolicy().hasHeightForWidth())
        self.uiCopyAccessTokenPushButton.setSizePolicy(sizePolicy)
        self.uiCopyAccessTokenPushButton.setObjectName("uiCopyAccessTokenPushButton")
        self.horizontalLayout.addWidget(self.uiCopyAccessTokenPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(UserPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(UserPreferencesPageWidget)

    def retranslateUi(self, UserPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        UserPreferencesPageWidget.setWindowTitle(_translate("UserPreferencesPageWidget", "User"))
        self.uiSettingsGroupBox.setTitle(_translate("UserPreferencesPageWidget", "Logged-in user information"))
        self.uiEmailLabel.setText(_translate("UserPreferencesPageWidget", "Email address:"))
        self.uiFullNameLabel.setText(_translate("UserPreferencesPageWidget", "Full name:"))
        self.uiUsernameLabel.setText(_translate("UserPreferencesPageWidget", "Username:"))
        self.uiChangePasswordPushButton.setText(_translate("UserPreferencesPageWidget", "&Change password"))
        self.uiCopyAccessTokenPushButton.setText(_translate("UserPreferencesPageWidget", "&Copy access token"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.qt import QtWidgets


class TreeWidgetItem(QtWidgets.QTreeWidgetItem):
    """
    QTreeWidgetItem reimplementation to allow numeric sort.
    """

    def __init__(self, parent=None):

        super().__init__(parent)

    def __lt__(self, other_item):

        column = self.treeWidget().sortColumn()
        try:
            return int(self.text(column)) < int(other_item.text(column))
        except ValueError:
            return self.text(column).lower() < other_item.text(column).lower()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/password_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PasswordDialog(object):
    def setupUi(self, PasswordDialog):
        PasswordDialog.setObjectName("PasswordDialog")
        PasswordDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        PasswordDialog.resize(377, 111)
        PasswordDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(PasswordDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.uiConfirmPasswordLineEdit = QtWidgets.QLineEdit(PasswordDialog)
        self.uiConfirmPasswordLineEdit.setMaxLength(100)
        self.uiConfirmPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiConfirmPasswordLineEdit.setObjectName("uiConfirmPasswordLineEdit")
        self.gridLayout.addWidget(self.uiConfirmPasswordLineEdit, 1, 1, 1, 1)
        self.uiPasswordLabel = QtWidgets.QLabel(PasswordDialog)
        self.uiPasswordLabel.setObjectName("uiPasswordLabel")
        self.gridLayout.addWidget(self.uiPasswordLabel, 0, 0, 1, 1)
        self.uiPasswordLineEdit = QtWidgets.QLineEdit(PasswordDialog)
        self.uiPasswordLineEdit.setMaxLength(100)
        self.uiPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiPasswordLineEdit.setObjectName("uiPasswordLineEdit")
        self.gridLayout.addWidget(self.uiPasswordLineEdit, 0, 1, 1, 1)
        self.uiConfirmPasswordLabel = QtWidgets.QLabel(PasswordDialog)
        self.uiConfirmPasswordLabel.setObjectName("uiConfirmPasswordLabel")
        self.gridLayout.addWidget(self.uiConfirmPasswordLabel, 1, 0, 1, 1)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(PasswordDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.gridLayout.addWidget(self.uiButtonBox, 2, 0, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 3, 0, 1, 1)

        self.retranslateUi(PasswordDialog)
        self.uiButtonBox.accepted.connect(PasswordDialog.accept) # type: ignore
        self.uiButtonBox.rejected.connect(PasswordDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(PasswordDialog)
        PasswordDialog.setTabOrder(self.uiPasswordLineEdit, self.uiConfirmPasswordLineEdit)

    def retranslateUi(self, PasswordDialog):
        _translate = QtCore.QCoreApplication.translate
        PasswordDialog.setWindowTitle(_translate("PasswordDialog", "Change password"))
        self.uiPasswordLabel.setText(_translate("PasswordDialog", "New password:"))
        self.uiConfirmPasswordLabel.setText(_translate("PasswordDialog", "Confirm password:"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from gns3.node import Node

import logging
log = logging.getLogger(__name__)


class Nat(Node):

    """
    Nat node

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """

    URL_PREFIX = "nat"

    def __init__(self, module, server, project):

        super().__init__(module, server, project)
        self.setStatus(Node.started)
        self._always_on = True
        self._nat_settings = {}
        self.settings().update(self._nat_settings)

    def info(self):
        """
        Returns information about this nat.

        :returns: formatted string
        """

        info = """NAT node {name} is always-on
  Running on server {host} with port {port}
""".format(name=self.name(),
           host=self.compute().name(),
           port=self.compute().port())

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "   Port {} is empty\n".format(port.name())
            else:
                port_info += "   Port {name} {description}\n".format(name=port.name(),
                                                                     description=port.description())

        return info + port_info

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this nat.

        :returns: symbol path (or resource).
        """

        return ":/symbols/nat.svg"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node categories
        """

        return [Node.end_devices]

    def __str__(self):

        return "Nat"



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import json
import os

from gns3.registry.config import Config
from gns3.settings import CONTROLLER_SETTINGS
from gns3.registry.appliance_to_template import ApplianceToTemplate


@pytest.fixture(scope="function")
def empty_config(tmpdir, images_dir, symbols_dir, local_config):
    settings = local_config.loadSectionSettings("Controller", CONTROLLER_SETTINGS)
    settings["images_path"] = images_dir
    settings["symbols_path"] = symbols_dir
    local_config.saveSectionSettings("Controller", settings)
    config = {
        "Servers": {
        },
        "Dynamips": {
            "allocate_aux_console_ports": False,
            "dynamips_path": "/Applications/GNS3.app/Contents/MacOS/dynamips",
            "ghost_ios_support": True,
            "mmap_support": True,
            "routers": [
            ],
            "sparse_memory_support": True,
            "use_local_server": True
        },
        "IOU": {
            "devices": [
            ],
            "license_check": True,
            "use_local_server": False
        },
        "Qemu": {
            "use_local_server": True,
            "vms": [
            ]
        },
        "Docker": {
            "containers": []
        }
    }
    path = str(tmpdir / "config")
    with open(path, "w+", encoding="utf-8") as f:
        json.dump(config, f)
    return Config(path)


@pytest.mark.parametrize(
    "appliance_file",
    [
        "cisco-iou-l3.gns3a",
        "cisco-iou-l3-v8.gns3a"
    ]
)
def test_add_appliance_iou(iou_l3, appliance_file):
    with open("tests/registry/appliances/{}".format(appliance_file), encoding="utf-8") as f:
        config = json.load(f)
    config["images"] = [
        {
            "type": "image",
            "filename": "i86bi-linux-l3-adventerprisek9-15.4.1T.bin",
            "path": iou_l3
        }
    ]
    new_template = ApplianceToTemplate().new_template(config, "local", "15.4.1T")
    assert new_template == {
        "category": "router",
        "template_type": "iou",
        "symbol": "router",
        "compute_id": "local",
        "name": "Cisco IOU L3",
        "nvram": 128,
        "ram": 256,
        "serial_adapters": 2,
        "ethernet_adapters": 2,
        "startup_config": "iou_l3_base_startup-config.txt",
        "image": os.path.basename(iou_l3),
        "path": os.path.basename(iou_l3)
    }


@pytest.mark.parametrize(
    "appliance_file",
    [
        "openvswitch.gns3a",
        "openvswitch-v8.gns3a"
    ]
)
def test_add_appliance_docker(appliance_file):
    with open("tests/registry/appliances/{}".format(appliance_file), encoding="utf-8") as f:
        config = json.load(f)

    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template == {
        "name": "Open vSwitch",
        "template_type": "docker",
        "image": "gns3/openvswitch:latest",
        "category": "switch",
        "symbol": "multilayer_switch",
        "compute_id": "local",
        "adapters": 16,
        "usage": "By default all interfaces are connected to the br0"
    }


@pytest.mark.parametrize(
    "appliance_file",
    [
        "cisco-3745.gns3a",
        "cisco-3745-v8.gns3a"
    ]
)
def test_add_appliance_dynamips(cisco_3745, appliance_file):
    with open("tests/registry/appliances/{}".format(appliance_file), encoding="utf-8") as f:
        config = json.load(f)
    config["images"] = [
        {
            "type": "image",
            "filename": "c3745-adventerprisek9-mz.124-25d.image",
            "path": cisco_3745,
            "idlepc": "0x60aa1da0"
        }
    ]

    new_template = ApplianceToTemplate().new_template(config, "local", "124-25d")
    assert new_template == {
        "template_type": "dynamips",
        "category": "router",
        "chassis": "",
        "idlepc": "0x60aa1da0",
        "image": "c3745-adventerprisek9-mz.124-25d.image",
        "name": "Cisco 3745",
        "nvram": 256,
        "platform": "c3745",
        "ram": 256,
        "compute_id": "local",
        "slot0": "GT96100-FE",
        "slot1": "NM-1FE-TX",
        "slot2": "NM-4T",
        "slot3": "",
        "slot4": "",
        "startup_config": "ios_base_startup-config.txt",
        "symbol": "router",
        "wic0": "WIC-1T",
        "wic1": "WIC-1T",
        "wic2": "WIC-1T"
    }


def test_add_appliance_guest(linux_microcore_img):
    with open("tests/registry/appliances/microcore-linux.gns3a", encoding="utf-8") as f:
        config = json.load(f)
    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "linux-microcore-3.4.1.img",
            "path": linux_microcore_img
        }
    ]

    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template == {
        "template_type": "qemu",
        "adapter_type": "e1000",
        "adapters": 1,
        "category": "guest",
        "console_type": "telnet",
        "symbol": "qemu_guest",
        "hda_disk_image": "linux-microcore-3.4.1.img",
        "name": "Micro Core Linux",
        "usage": "Just start the appliance",
        "ram": 32,
        "compute_id": "local"
    }


def test_add_appliance_with_symbol(linux_microcore_img):
    with open("tests/registry/appliances/microcore-linux.gns3a", encoding="utf-8") as f:
        config = json.load(f)
    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "linux-microcore-3.4.1.img",
            "path": linux_microcore_img
        }
    ]
    config["symbol"] = ":/symbols/asa.svg"
    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template["symbol"] == ":/symbols/asa.svg"


def test_add_appliance_with_symbol_from_symbols_dir(linux_microcore_img, symbols_dir, controller):

    with open("tests/registry/appliances/microcore-linux.gns3a", encoding="utf-8") as f:
        config = json.load(f)
    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "linux-microcore-3.4.1.img",
            "path": linux_microcore_img
        }
    ]
    config["symbol"] = "linux_guest.svg"

    controller.settings()["symbols_path"] = symbols_dir
    symbol_path = os.path.join(symbols_dir, "linux_guest.svg")
    open(symbol_path, 'w+').close()

    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template["symbol"] == "linux_guest.svg"


def test_add_appliance_with_port_name_format(linux_microcore_img):
    with open("tests/registry/appliances/microcore-linux.gns3a", encoding="utf-8") as f:
        config = json.load(f)
    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "linux-microcore-3.4.1.img",
            "path": linux_microcore_img
        }
    ]
    config["port_name_format"] = "eth{0}"
    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template["port_name_format"] == "eth{0}"


def test_add_appliance_with_boot_priority(linux_microcore_img):
    with open("tests/registry/appliances/microcore-linux.gns3a", encoding="utf-8") as f:
        config = json.load(f)
    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "linux-microcore-3.4.1.img",
            "path": linux_microcore_img
        }
    ]
    config["qemu"]["boot_priority"] = "dc"
    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template["boot_priority"] == "dc"


@pytest.mark.parametrize(
    "appliance_file",
    [
        "arista-veos.gns3a",
        "arista-veos-v8.gns3a"
    ]
)
def test_add_appliance_router_two_disk(images_dir, appliance_file):
    with open("tests/registry/appliances/{}".format(appliance_file), encoding="utf-8") as f:
        config = json.load(f)

    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "a",
            "path": os.path.join(images_dir, "QEMU", "a")
        },
        {
            "type": "hdb_disk_image",
            "filename": "b",
            "path": os.path.join(images_dir, "QEMU", "b")
        }
    ]

    new_template = ApplianceToTemplate().new_template(config, "local", "4.13.8M")
    expected_result = {
        "template_type": "qemu",
        "adapter_type": "e1000",
        "adapters": 8,
        "category": "router",
        "symbol": "router",
        "hda_disk_image": "a",
        "hdb_disk_image": "b",
        "name": "Arista vEOS",
        "ram": 2048,
        "console_type": "telnet",
        "compute_id": "local"
    }
    if "v8" in appliance_file:
        expected_result["platform"] = "x86_64"  # platform was added in v8
    assert new_template == expected_result


def test_add_appliance_v8_default_properties_inheritance(images_dir):
    with open("tests/registry/appliances/empty-vm-v8.gns3a", encoding="utf-8") as f:
        config = json.load(f)

    # check that default properties are used
    new_template = ApplianceToTemplate().new_template(config, "local", "8G")
    expected_result = {
        "name": "Empty VM",
        "template_type": "qemu",
        "symbol": "qemu_guest",
        "category": "guest",
        "adapter_type": "e1000",
        "adapters": 1,
        "ram": 1024,
        "hda_disk_interface": "sata",
        "platform": "x86_64",
        "console_type": "vnc",
        "boot_priority": "d",
        "compute_id": "local",
        "usage": "Default at first boot the VM will start from the cdrom."
    }
    assert new_template == expected_result

    # check that specific properties are used along with default properties
    new_template = ApplianceToTemplate().new_template(config, "local", "30G")
    expected_result.update(
        {
            "adapters": 8,
            "platform": "i386",
        }
    )
    assert new_template == expected_result

    # check that specific properties are used along without default properties
    new_template = ApplianceToTemplate().new_template(config, "local", "100G")
    expected_result = {
        "name": "Empty VM",
        "template_type": "qemu",
        "symbol": "qemu_guest",
        "category": "guest",
        "ram": 512,
        "platform": "arm",
        "compute_id": "local",
        "usage": "Default at first boot the VM will start from the cdrom."
    }
    assert new_template == expected_result

    # check that specific properties are used with "usage", "symbol" and "category" defined at the version level
    new_template = ApplianceToTemplate().new_template(config, "local", "200G")
    expected_result.update(
        {
            "usage": "This is how to use this version",
            "symbol": "ethernet_switch",
            "category": "switch"
        }
    )
    assert new_template == expected_result


def test_add_appliance_path_relative_to_images_dir(tmpdir, linux_microcore_img):
    with open("tests/registry/appliances/microcore-linux.gns3a", encoding="utf-8") as f:
        config = json.load(f)

    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "linux-microcore-3.4.1.img",
            "path": linux_microcore_img
        }
    ]

    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template["hda_disk_image"] == "linux-microcore-3.4.1.img"


def test_add_appliance_path_non_relative_to_images_dir(tmpdir, images_dir):
    with open("tests/registry/appliances/microcore-linux.gns3a", encoding="utf-8") as f:
        config = json.load(f)

    config["images"] = [
        {
            "type": "hda_disk_image",
            "filename": "a.img",
            "path": str(tmpdir / "a.img")
        }
    ]
    with open(str(tmpdir / "a.img"), "w+") as f:
        f.write("a")
    new_template = ApplianceToTemplate().new_template(config, "local")
    assert new_template["hda_disk_image"] == "a.img"


def test_relative_image_path(images_dir, tmpdir):

    # Image in image directory no need to copy it
    open(os.path.join(images_dir, "QEMU", "a"), "w+").close()
    assert ApplianceToTemplate()._relative_image_path("QEMU", os.path.join(images_dir, "QEMU", "a")) == "a"

    # Image in image directory no need to copy it but with a different file name
    open(os.path.join(images_dir, "QEMU", "a"), "w+").close()
    assert ApplianceToTemplate()._relative_image_path("QEMU", os.path.join(images_dir, "QEMU", "a")) == "a"

    # Image outside image directory we need to copy it
    open(str(tmpdir / "b"), "w+").close()
    assert ApplianceToTemplate()._relative_image_path("QEMU", str(tmpdir / "b")) == "b"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for QEMU preferences.
"""

from gns3.qt import QtWidgets
from .. import Qemu
from ..ui.qemu_preferences_page_ui import Ui_QemuPreferencesPageWidget
from ..settings import QEMU_SETTINGS


class QemuPreferencesPage(QtWidgets.QWidget, Ui_QemuPreferencesPageWidget):
    """
    QWidget preference page for QEMU.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        # connect signals
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiEnableHardwareAccelerationCheckBox.stateChanged.connect(self._hardwareAccelerationSlot)

    def _hardwareAccelerationSlot(self, state):
        """
        Slot to enable or not the require KVM acceleration check box.
        """

        if state:
            self.uiRequireHardwareAccelerationCheckBox.setEnabled(True)
            self.uiRequireHardwareAccelerationCheckBox.setChecked(True)
        else:
            self.uiRequireHardwareAccelerationCheckBox.setEnabled(False)
            self.uiRequireHardwareAccelerationCheckBox.setChecked(False)

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(QEMU_SETTINGS)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: QEMU settings
        """

        self.uiEnableHardwareAccelerationCheckBox.setChecked(settings["enable_hardware_acceleration"])
        self.uiRequireHardwareAccelerationCheckBox.setChecked(settings["require_hardware_acceleration"])

    def loadPreferences(self):
        """
        Loads QEMU preferences.
        """

        qemu_settings = Qemu.instance().settings()
        self._populateWidgets(qemu_settings)

    def savePreferences(self):
        """
        Saves QEMU preferences.
        """

        new_settings = {"enable_hardware_acceleration": self.uiEnableHardwareAccelerationCheckBox.isChecked(),
                        "require_hardware_acceleration": self.uiRequireHardwareAccelerationCheckBox.isChecked()}
        Qemu.instance().setSettings(new_settings)
